package de.jave.eclipse.poormans.quickfixes;

import java.util.ArrayList;
import java.util.List;

import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;

import org.eclipse.jdt.core.compiler.IProblem;

public class DefaultQuickFixFactory {

  public List<RegExpQuickFix> createDefaultQuickFixes() {
    return new ArrayList<RegExpQuickFix>() {
      {
        //values.length -> values.size()
        add(new RegExpQuickFix(IProblem.UndefinedField, "(.*)\\.length", //$NON-NLS-1$
            "{1}.size()", //$NON-NLS-1$
            "Change to List 'size()'", //$NON-NLS-1$
            RegExpQuickFix.DEFAULT_RELEVANCE));

        //values[i] -> values.get(i)
        add(new RegExpQuickFix(IProblem.ArrayReferenceRequired, "(.*)\\[(.*)\\]", //$NON-NLS-1$
            "{1}.get({2})", //$NON-NLS-1$
            "Change to List '.get(...)'", //$NON-NLS-1$
            RegExpQuickFix.DEFAULT_RELEVANCE));

        //values.get(i) -> values[i]
        add(new RegExpQuickFix(IProblem.NoMessageSendOnArrayType, "(.*)\\.get\\((.*)\\)", //$NON-NLS-1$
            "{1}[{2}]", //$NON-NLS-1$
            "Change to array index '[...]'", //$NON-NLS-1$
            RegExpQuickFix.DEFAULT_RELEVANCE));

        //return; -> return null;
        // -> Already covered by Eclipse's "Change return" quick fix.
        //        add(new RegExpQuickFix(IProblem.ShouldReturnValue, "(return)", //$NON-NLS-1$
        //            "{1} null", //$NON-NLS-1$
        //            "Change to return null", //$NON-NLS-1$
        //            RegExpQuickFix.DEFAULT_RELEVANCE));

        //Foo(); -> new Foo();
        add(new RegExpQuickFix(IProblem.UndefinedMethod, "(.*)", "new {1}", "Change to constructor call", 1)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      }
    };
  }
}