package de.jave.eclipse.poormans.quickfixes.preferences;

import java.util.ArrayList;
import java.util.List;

import de.jave.eclipse.poormans.quickfixes.DefaultQuickFixFactory;
import de.jave.eclipse.poormans.quickfixes.PoorMansQuickFixesPlugin;
import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

public class PoorMansQuickFixesPreferencesInitializer extends AbstractPreferenceInitializer {

  private static final String PATTERN_STRING = "patternString"; //$NON-NLS-1$
  private static final String NAME_FORMAT = "nameFormat"; //$NON-NLS-1$
  private static final String PROBLEM_ID = "problemId"; //$NON-NLS-1$
  private static final String RELEVANCE = "relevance"; //$NON-NLS-1$
  private static final String REPLACEMENT_FORMAT = "replacementFormat"; //$NON-NLS-1$
  private static final String REG_EXP_QUICK_FIX = "RegExpQuickFix"; //$NON-NLS-1$
  private static final String REG_EXP_QUICK_FIX_COUNT = "RegExpQuickFixCount"; //$NON-NLS-1$

  @Override
  public void initializeDefaultPreferences() {
    final IPreferenceStore store = PoorMansQuickFixesPlugin.getDefault().getPreferenceStore();
    final List<RegExpQuickFix> quickFixes = new DefaultQuickFixFactory().createDefaultQuickFixes();
    store.setDefault(REG_EXP_QUICK_FIX_COUNT, quickFixes.size());
    for (int i = 0; i < quickFixes.size(); ++i) {
      final RegExpQuickFix fix = quickFixes.get(i);
      final String prefix = REG_EXP_QUICK_FIX + i + "."; //$NON-NLS-1$
      store.setDefault(prefix + REPLACEMENT_FORMAT, fix.getReplacementFormat());
      store.setDefault(prefix + PROBLEM_ID, fix.getProblemId());
      store.setDefault(prefix + NAME_FORMAT, fix.getNameFormat());
      store.setDefault(prefix + PATTERN_STRING, fix.getPatternString());
      store.setDefault(prefix + RELEVANCE, fix.getRelevance());
    }
  }

  public static void store(final List<RegExpQuickFix> quickFixes, final IPreferenceStore preferenceStore) {
    preferenceStore.setValue(REG_EXP_QUICK_FIX_COUNT, quickFixes.size());
    for (int i = 0; i < quickFixes.size(); ++i) {
      final RegExpQuickFix fix = quickFixes.get(i);
      final String prefix = REG_EXP_QUICK_FIX + i + "."; //$NON-NLS-1$
      preferenceStore.setValue(prefix + REPLACEMENT_FORMAT, fix.getReplacementFormat());
      preferenceStore.setValue(prefix + PROBLEM_ID, fix.getProblemId());
      preferenceStore.setValue(prefix + NAME_FORMAT, fix.getNameFormat());
      preferenceStore.setValue(prefix + PATTERN_STRING, fix.getPatternString());
      preferenceStore.setValue(prefix + RELEVANCE, fix.getRelevance());
    }
  }

  public static List<RegExpQuickFix> load(final IPreferenceStore preferenceStore) {
    final int count = preferenceStore.getInt(REG_EXP_QUICK_FIX_COUNT);
    final ArrayList<RegExpQuickFix> list = new ArrayList<RegExpQuickFix>();
    for (int i = 0; i < count; ++i) {
      final String prefix = REG_EXP_QUICK_FIX + i + "."; //$NON-NLS-1$
      final String replacementFormat = preferenceStore.getString(prefix + REPLACEMENT_FORMAT);
      final int problemId = preferenceStore.getInt(prefix + PROBLEM_ID);
      final String nameFormat = preferenceStore.getString(prefix + NAME_FORMAT);
      final String patternString = preferenceStore.getString(prefix + PATTERN_STRING);

      final String relevanceString = preferenceStore.getString(prefix + RELEVANCE);
      final int relevance;
      if (relevanceString.length() == 0) {
        relevance = RegExpQuickFix.DEFAULT_RELEVANCE;
      }
      else {
        relevance = preferenceStore.getInt(prefix + RELEVANCE);
      }
      list.add(new RegExpQuickFix(problemId, patternString, replacementFormat, nameFormat, relevance));
    }
    return list;
  }
}