package de.jave.eclipse.poormans.quickfixes.preferences;

import java.util.ArrayList;
import java.util.Arrays;

import de.jave.eclipse.poormans.quickfixes.DefaultQuickFixFactory;
import de.jave.eclipse.poormans.quickfixes.PoorMansQuickFixesPlugin;
import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;
import de.jave.eclipse.poormans.quickfixes.util.ChangeEvent;
import de.jave.eclipse.poormans.quickfixes.util.IChangeListener;
import de.jave.eclipse.poormans.quickfixes.util.ObjectModel;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

public class PoorMansQuickFixesPreferencesPage extends PreferencePage implements IWorkbenchPreferencePage {

  private List list;
  private Button fImportButton;
  private Button fExportButton;

  private final ObjectModel<RegExpQuickFix> selectedQuickFixModel = new ObjectModel<RegExpQuickFix>();
  private final RegExpQuickFixDetailsPanel detailsPanel = new RegExpQuickFixDetailsPanel(selectedQuickFixModel);

  public PoorMansQuickFixesPreferencesPage() {
    //nothing to do
  }

  public PoorMansQuickFixesPreferencesPage(String title) {
    super(title);
    //nothing to do
  }

  public PoorMansQuickFixesPreferencesPage(String title, ImageDescriptor image) {
    super(title, image);
    //nothing to do
  }

  @Override
  protected Control createContents(Composite parent) {
    Composite comp = new Composite(parent, SWT.NONE);
    comp.setLayout(new GridLayout(1, true));
    createDescriptionText(comp).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    createListContents(comp).setLayoutData(new GridData(GridData.FILL_BOTH));
    createCurrentItemContents(comp).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    return comp;
  }

  private Control createDescriptionText(Composite comp) {
    Label label = new Label(comp, SWT.WRAP);
    label.setText("Poor man's quick fixes are Java problem quick fixes based on a problem ID, a regular\r\n"
        + "expression matching the problem code and a messageformat string for creating\r\n"
        + "the replacement text.");
    return label;
  }

  private Control createCurrentItemContents(Composite parent) {
    return detailsPanel.create(parent);
  }

  private void updateCurrentItemFromSelectionModel() {
    int[] indices = list.getSelectionIndices();
    if (indices.length != 1) {
      return;
    }
    int index = indices[0];
    quickFixes.set(index, selectedQuickFixModel.getValue());
    String displayName = getDisplayName(quickFixes.get(index));
    if (!list.getItem(index).equals(displayName)) {
      list.setItem(index, displayName);
    }
  }

  private java.util.List<RegExpQuickFix> quickFixes;

  private Control createListContents(Composite parent) {
    Composite comp = new Composite(parent, SWT.NONE);
    GridLayout gridLayout = new GridLayout(1, false);
    gridLayout.marginHeight = 0;
    gridLayout.marginWidth = 0;
    comp.setLayout(gridLayout);

    Label label = new Label(comp, SWT.LEFT);
    label.setText("&Quick fixes:");

    Composite comp1 = new Composite(comp, SWT.NONE);
    comp1.setLayoutData(new GridData(GridData.FILL_BOTH));
    GridLayout gridLayout1 = new GridLayout(2, false);
    gridLayout1.marginHeight = 0;
    gridLayout1.marginWidth = 0;
    comp1.setLayout(gridLayout1);

    list = new List(comp1, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL);
    list.setLayoutData(new GridData(GridData.FILL_BOTH));

    PoorMansQuickFixesPlugin plugin = PoorMansQuickFixesPlugin.getDefault();
    quickFixes = new ArrayList<RegExpQuickFix>(plugin.getRegExpQuickFixes());

    for (int i = 0; i < quickFixes.size(); ++i) {
      list.add(getDisplayName(quickFixes.get(i)));
    }

    list.addSelectionListener(new SelectionListener() {
      public void widgetSelected(SelectionEvent e) {
        updateModelFromSelection();
      }

      public void widgetDefaultSelected(SelectionEvent e) {
        //nothing to do
      }
    });

    selectedQuickFixModel.addChangeListener(new IChangeListener() {
      public void stateChanged(ChangeEvent e) {
        updateCurrentItemFromSelectionModel();
      }
    });
    Composite buttonPanel = new Composite(comp1, SWT.NONE);
    buttonPanel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
    RowLayout rowLayout2 = new RowLayout(SWT.VERTICAL);
    rowLayout2.wrap = false;
    rowLayout2.pack = false;
    rowLayout2.marginTop = 0;
    rowLayout2.marginLeft = 0;
    rowLayout2.marginRight = 0;
    buttonPanel.setLayout(rowLayout2);

    Button bNew = new Button(buttonPanel, SWT.PUSH);
    bNew.setText("Add");
    bNew.addSelectionListener(new SelectionListener() {
      public void widgetSelected(SelectionEvent event) {
        doCreateNewQuickFixEntry();
      }

      public void widgetDefaultSelected(SelectionEvent event) {
        //nothing to do
      }
    });
    final Button bDuplicate = new Button(buttonPanel, SWT.PUSH);
    bDuplicate.setText("Duplicate");
    bDuplicate.addSelectionListener(new SelectionListener() {
      public void widgetSelected(SelectionEvent event) {
        doDuplicateLabelDecoration();
      }

      public void widgetDefaultSelected(SelectionEvent event) {
        //nothing to do
      }
    });
    final Button bRemove = new Button(buttonPanel, SWT.PUSH);
    bRemove.setText("Remove");
    bRemove.addSelectionListener(new SelectionListener() {
      public void widgetSelected(SelectionEvent event) {
        doRemoveCurrentQuickFix();
      }

      public void widgetDefaultSelected(SelectionEvent event) {
        //nothing to do
      }
    });
    selectedQuickFixModel.addChangeListener(new IChangeListener() {
      public void stateChanged(ChangeEvent e) {
        updateButtonsEnabled(bRemove, bDuplicate);
      }
    });

    createSeparator(buttonPanel);

    fImportButton = new Button(buttonPanel, SWT.PUSH);
    fImportButton.setText("&Import...");
    fImportButton.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event e) {
        doImport();
      }
    });

    fExportButton = new Button(buttonPanel, SWT.PUSH);
    fExportButton.setText("Export...");
    fExportButton.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event e) {
        doExport();
      }
    });

    updateButtonsEnabled(bRemove, bDuplicate);
    return comp;
  }

  private void doImport() {
    RegExpQuickFix[] fixes = RegExpQuickFixesExporterImporter.doImport(getShell());
    if (fixes == null) {
      return;
    }
    int addedCount = 0;
    for (RegExpQuickFix fix : fixes) {
      if (!quickFixes.contains(fix)) {
        addQuickFix(fix);
        ++addedCount;
      }
    }
    if (addedCount == 0) {
      String title = "Import";
      String message = "The file did not contain any quick fixes that did not already exist.";
      MessageDialog.openInformation(getShell(), title, message);
    }
    else {
      String title = "Import";
      String message = addedCount + " quick fixes have been imported and added to the end of the list.";
      MessageDialog.openInformation(getShell(), title, message);
    }
  }

  private void doExport() {
    int[] indices = list.getSelectionIndices();
    RegExpQuickFix[] fixes = new RegExpQuickFix[indices.length];
    for (int i = 0; i < indices.length; ++i) {
      fixes[i] = quickFixes.get(indices[i]);
    }
    RegExpQuickFixesExporterImporter.doExport(getShell(), fixes);
  }

  private Label createSeparator(Composite parent) {
    Label separator = new Label(parent, SWT.NONE);
    separator.setVisible(false);
    separator.setLayoutData(new RowData(4, 4));
    return separator;
  }

  private void updateButtonsEnabled(Button removeButton, Button duplicateButton) {
    int selectionCount = list.getSelectionCount();
    removeButton.setEnabled(selectionCount > 0);
    duplicateButton.setEnabled(selectionCount == 1);
    fExportButton.setEnabled(selectionCount > 0);
  }

  private String getDisplayName(RegExpQuickFix quickFix) {
    final String name = quickFix.getNameFormat();
    return name == null || name.length() == 0 ? "..." : name; //$NON-NLS-1$
  }

  private void doCreateNewQuickFixEntry() {
    RegExpQuickFix newQuickFix = new RegExpQuickFix(-1, "", "", "", RegExpQuickFix.DEFAULT_RELEVANCE);
    addQuickFix(newQuickFix);
  }

  private void doDuplicateLabelDecoration() {
    RegExpQuickFix value = selectedQuickFixModel.getValue();
    RegExpQuickFix clone = value.clone();
    addQuickFix(clone);
  }

  @Override
  protected void performDefaults() {
    quickFixes.clear();
    list.removeAll();
    java.util.List<RegExpQuickFix> defaultDecorations = new DefaultQuickFixFactory().createDefaultQuickFixes();
    for (RegExpQuickFix quickFix : defaultDecorations) {
      addQuickFix(quickFix);
    }
    super.performDefaults();
  }

  private void addQuickFix(RegExpQuickFix newQuickFix) {
    quickFixes.add(newQuickFix);
    list.add(getDisplayName(newQuickFix));
    list.setSelection(quickFixes.size() - 1);
    selectedQuickFixModel.setValue(newQuickFix);
    detailsPanel.requestFocus();
  }

  private void doRemoveCurrentQuickFix() {
    int[] selectionIndices = list.getSelectionIndices();
    Arrays.sort(selectionIndices);
    for (int i = selectionIndices.length - 1; i >= 0; --i) {
      int index = selectionIndices[i];
      list.remove(index);
      quickFixes.remove(index);
      list.setSelection(index - 1);
    }
    updateModelFromSelection();
  }

  public void init(IWorkbench workbench) {
    //nothing to do
  }

  private void updateModelFromSelection() {
    int[] indices = list.getSelectionIndices();
    if (indices.length == 1) {
      selectedQuickFixModel.setValue(quickFixes.get(indices[0]));
    }
    else {
      selectedQuickFixModel.setValue(null);
    }
  }

  @Override
  public boolean performOk() {
    PoorMansQuickFixesPlugin.getDefault().setQuickFixes(new ArrayList<RegExpQuickFix>(quickFixes));
    return true;
  }
}