package de.jave.eclipse.poormans.quickfixes.preferences;

import java.util.List;

import de.jave.eclipse.poormans.quickfixes.util.ProblemIdConstant;
import de.jave.eclipse.poormans.quickfixes.util.ProblemIdFactory;

import org.eclipse.jdt.core.compiler.IProblem;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

public class ProblemIdSelectionDialog extends Dialog {

  private final List<ProblemIdConstant> constants;
  private Combo combo;
  private int selectedProblemId;

  public ProblemIdSelectionDialog(Shell shell, int selectedProblemId) {
    super(shell);
    this.selectedProblemId = selectedProblemId;
    this.constants = ProblemIdFactory.getProblemIdConstants();
  }

  /* (non-Javadoc)
   * Method declared in Window.
   */
  @Override
  protected void configureShell(Shell shell) {
    super.configureShell(shell);
    shell.setText("Select Problem ID");
  }

  @Override
  protected Control createDialogArea(Composite parent) {
    Composite comp = new Composite(parent, SWT.NONE | SWT.WRAP);
    org.eclipse.swt.layout.GridLayout layout = new GridLayout(1, false);
    comp.setLayout(layout);

    Label label1 = new Label(comp, SWT.NONE);
    label1.setText("Problem IDs as defined in the class " + IProblem.class.getName() + ":");

    createProblemIdSelectionCombo(comp);

    Label label2 = new Label(comp, SWT.NONE | SWT.WRAP);
    label2.setText("Note that you can find out problem IDs by right clicking in a Java editor"
        + " near a problem.\r\nJust select the context menu item 'Show problem ID'.");

    return comp;
  }

  private Control createProblemIdSelectionCombo(Composite parent) {
    Composite comp = new Composite(parent, SWT.NONE);
    comp.setLayout(new GridLayout(2, false));

    Label label = new Label(comp, SWT.NONE);
    label.setText("Problem ID:");//i18n

    combo = new Combo(comp, SWT.DROP_DOWN | SWT.READ_ONLY);
    int selectionIndex = -1;
    for (int i = 0; i < constants.size(); ++i) {
      ProblemIdConstant constant = constants.get(i);
      combo.add(constant.getName() + " (" + constant.getValue() + ")");
      if (constant.getValue() == selectedProblemId) {
        selectionIndex = i;
      }
    }
    combo.select(selectionIndex);
    combo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    return comp;
  }

  protected void okPressed() {
    final int selectionComboIndex = combo.getSelectionIndex();
    selectedProblemId = constants.get(selectionComboIndex).getValue();
    super.okPressed();
  }

  public int getSelectedProblemId() {
    return selectedProblemId;
  }
}