package de.jave.eclipse.poormans.quickfixes.preferences;

import java.util.ArrayList;
import java.util.List;

import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;
import de.jave.eclipse.poormans.quickfixes.util.ChangeEvent;
import de.jave.eclipse.poormans.quickfixes.util.IChangeListener;
import de.jave.eclipse.poormans.quickfixes.util.ObjectModel;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

public class RegExpQuickFixDetailsPanel {

  private final ObjectModel<RegExpQuickFix> selectedQuickFixModel;
  private final List<Control> enableTogglingControls = new ArrayList<Control>();
  private Spinner spinnerProblemId;
  private Spinner spinnerRelevance;
  private Text tfPatternString;
  private Text tfNameFormat;
  private Text tfFixFormat;

  public RegExpQuickFixDetailsPanel(final ObjectModel<RegExpQuickFix> selectedQuickFixModel) {
    this.selectedQuickFixModel = selectedQuickFixModel;
  }

  public Control create(final Composite parent) {
    final Composite comp = new Composite(parent, SWT.NONE);
    final GridLayout gridLayout = new GridLayout(3, false);
    gridLayout.marginHeight = 0;
    gridLayout.marginWidth = 0;
    comp.setLayout(gridLayout);

    addLabel(comp, "&Name:");
    tfNameFormat = new Text(comp, SWT.BORDER);
    enableTogglingControls.add(tfNameFormat);
    tfNameFormat.setLayoutData(createHorizontalFillData(2));
    tfNameFormat.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        final RegExpQuickFix value = selectedQuickFixModel.getValue();
        if (value == null) {
          return;
        }
        final String nameFormat = tfNameFormat.getText();
        final String fixFormat = value.getReplacementFormat();
        final String patternString = value.getPatternString();
        final int problemId = value.getProblemId();
        final int relevance = value.getRelevance();
        selectedQuickFixModel.setValue(new RegExpQuickFix(
            problemId,
            patternString,
            fixFormat,
            nameFormat,
            relevance));
      }
    });
    tfNameFormat.setToolTipText("Messageformat string defining the display text when the quick fix is\r\n"
        + "presented to the user.\r\n"
        + "You can insert groups {0..} of the original text from the regular expression pattern.");

    addLabel(comp, "Java Problem &ID:");

    spinnerProblemId = new Spinner(comp, SWT.BORDER);
    spinnerProblemId.setToolTipText("Problem ID as defined by the Eclipse JDT");
    spinnerProblemId.setMaximum(Integer.MAX_VALUE);
    spinnerProblemId.setMinimum(0);
    spinnerProblemId.setIncrement(1);

    enableTogglingControls.add(spinnerProblemId);
    spinnerProblemId.setLayoutData(createHorizontalFillData(1));
    spinnerProblemId.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        final RegExpQuickFix value = selectedQuickFixModel.getValue();
        if (value == null) {
          return;
        }
        final String nameFormat = value.getNameFormat();
        final String fixFormat = value.getReplacementFormat();
        final String patternString = value.getPatternString();
        final int problemId = spinnerProblemId.getSelection();
        final int relevance = value.getRelevance();
        selectedQuickFixModel.setValue(new RegExpQuickFix(
            problemId,
            patternString,
            fixFormat,
            nameFormat,
            relevance));
      }
    });

    final Button button = new Button(comp, SWT.PUSH);
    enableTogglingControls.add(button);
    button.setText("&Select...");

    button.addSelectionListener(new SelectionListener() {
      public void widgetSelected(final SelectionEvent e) {
        final ProblemIdSelectionDialog dialog = new ProblemIdSelectionDialog(button.getShell(), spinnerProblemId
            .getSelection());
        final int returnCode = dialog.open();
        if (returnCode == Window.CANCEL) {
          return;
        }
        final int problemId = dialog.getSelectedProblemId();
        spinnerProblemId.setSelection(problemId);
      }

      public void widgetDefaultSelected(final SelectionEvent e) {
        //nothing to do    
      }
    });

    addLabel(comp, "Relevance:");

    spinnerRelevance = new Spinner(comp, SWT.BORDER);
    spinnerRelevance
        .setToolTipText("Relevance of this completion proposal. Used to determine if this proposal is more relevant than another proposal.");
    spinnerRelevance.setMaximum(100);
    spinnerRelevance.setMinimum(0);
    spinnerRelevance.setIncrement(1);

    enableTogglingControls.add(spinnerRelevance);
    spinnerRelevance.setLayoutData(createHorizontalFillData(2));
    spinnerRelevance.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        final RegExpQuickFix value = selectedQuickFixModel.getValue();
        if (value == null) {
          return;
        }
        final String nameFormat = value.getNameFormat();
        final String fixFormat = value.getReplacementFormat();
        final String patternString = value.getPatternString();
        final int problemId = value.getProblemId();
        final int relevance = spinnerRelevance.getSelection();
        selectedQuickFixModel.setValue(new RegExpQuickFix(
            problemId,
            patternString,
            fixFormat,
            nameFormat,
            relevance));
      }
    });

    addLabel(comp, "Regular e&xpression:");
    tfPatternString = new Text(comp, SWT.BORDER);
    enableTogglingControls.add(tfPatternString);
    tfPatternString.setLayoutData(createHorizontalFillData(2));
    tfPatternString.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        final RegExpQuickFix value = selectedQuickFixModel.getValue();
        if (value == null) {
          return;
        }
        final String nameFormat = value.getNameFormat();
        final String fixFormat = value.getReplacementFormat();
        final String patternString = tfPatternString.getText();
        final int problemId = value.getProblemId();
        final int relevance = value.getRelevance();
        selectedQuickFixModel.setValue(new RegExpQuickFix(
            problemId,
            patternString,
            fixFormat,
            nameFormat,
            relevance));
      }
    });
    tfPatternString
        .setToolTipText("Regular expression matching the problem string. Define groups to capture parts\r\n"
            + "of the text in order to reuse then for replacement.");

    addLabel(comp, "R&eplacement format:");
    tfFixFormat = new Text(comp, SWT.BORDER);
    enableTogglingControls.add(tfFixFormat);
    tfFixFormat.setLayoutData(createHorizontalFillData(2));
    tfFixFormat.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        final RegExpQuickFix value = selectedQuickFixModel.getValue();
        if (value == null) {
          return;
        }
        final String nameFormat = value.getNameFormat();
        final String fixFormat = tfFixFormat.getText();
        final String patternString = value.getPatternString();
        final int problemId = value.getProblemId();
        final int relevance = value.getRelevance();
        selectedQuickFixModel.setValue(new RegExpQuickFix(
            problemId,
            patternString,
            fixFormat,
            nameFormat,
            relevance));
      }
    });
    tfFixFormat.setToolTipText("Messageformat string defining the replacement text when the quick fix is\r\n"
        + "applied.\r\n"
        + "You can insert groups {0..} of the original text from the regular expression pattern.");

    selectedQuickFixModel.addChangeListener(new IChangeListener() {
      public void stateChanged(final ChangeEvent e) {
        updateView();
      }
    });
    updateView();
    return comp;
  }

  private static GridData createHorizontalFillData(final int columnCount) {
    final GridData data = new GridData(GridData.FILL_HORIZONTAL);
    data.horizontalSpan = columnCount;
    return data;
  }

  private void addLabel(final Composite comp0, final String name) {
    final Label label1 = new Label(comp0, SWT.NONE);
    enableTogglingControls.add(label1);
    label1.setText(name);
  }

  private void updateView() {
    final RegExpQuickFix quickFix = selectedQuickFixModel.getValue();
    if (quickFix == null) {
      tfPatternString.setText(""); //$NON-NLS-1$
      tfNameFormat.setText(""); //$NON-NLS-1$
      tfFixFormat.setText(""); //$NON-NLS-1$
      spinnerProblemId.setSelection(0);
      spinnerRelevance.setSelection(RegExpQuickFix.DEFAULT_RELEVANCE);
    }
    else {
      setTextIfChanged(tfPatternString, quickFix.getPatternString());
      setTextIfChanged(tfNameFormat, quickFix.getNameFormat());
      setTextIfChanged(tfFixFormat, quickFix.getReplacementFormat());
      spinnerProblemId.setSelection(quickFix.getProblemId());
      spinnerRelevance.setSelection(quickFix.getRelevance());
    }
    setEnabled(quickFix != null);
  }

  private void setTextIfChanged(final Text text, final String string) {
    if (text.getText().equals(string)) {
      return;
    }
    text.setText(string);
  }

  private void setEnabled(final boolean enabled) {
    for (final Control control : enableTogglingControls) {
      control.setEnabled(enabled);
    }
  }

  public void requestFocus() {
    tfPatternString.selectAll();
    tfPatternString.forceFocus();
  }
}