package de.jave.eclipse.poormans.quickfixes.regexp;

import java.util.regex.Pattern;

public class RegExpQuickFix implements Cloneable {

  public static final int DEFAULT_RELEVANCE = 90;

  private final int problemId;
  private final String patternString;
  private final String replacementFormat;
  private final String nameFormat;
  private final int relevance;
  private Pattern pattern;

  public RegExpQuickFix(
      int problemId,
      String patternString,
      String replacementFormat,
      String nameFormat,
      int relevance) {
    this.problemId = problemId;
    this.patternString = patternString;
    this.replacementFormat = replacementFormat;
    this.nameFormat = nameFormat;
    this.relevance = relevance;
  }

  @Override
  public RegExpQuickFix clone() {
    try {
      return (RegExpQuickFix) super.clone();
    }
    catch (CloneNotSupportedException e) {
      throw new RuntimeException(e);
    }
  }

  @Override
  public boolean equals(Object obj) {
    if (!(obj instanceof RegExpQuickFix)) {
      return false;
    }
    RegExpQuickFix other = (RegExpQuickFix) obj;
    return problemId == other.problemId
        && patternString.equals(other.patternString)
        && replacementFormat.equals(other.replacementFormat)
        && nameFormat.equals(other.nameFormat)
        && relevance == other.relevance;
  }

  public int getProblemId() {
    return problemId;
  }

  public String getPatternString() {
    return patternString;
  }

  public Pattern getPattern() {
    if (pattern == null) {
      pattern = Pattern.compile(patternString);
    }
    return pattern;
  }

  public String getReplacementFormat() {
    return replacementFormat;
  }

  public String getNameFormat() {
    return nameFormat;
  }

  public int getRelevance() {
    return relevance;
  }
}