//Copyright (c) 2006 by disy Informationssysteme GmbH
package de.jave.eclipse.poormans.quickfixes.regexp;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.jave.eclipse.poormans.quickfixes.PoorMansQuickFixesPlugin;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.text.java.IInvocationContext;
import org.eclipse.jdt.ui.text.java.IJavaCompletionProposal;
import org.eclipse.jdt.ui.text.java.IProblemLocation;
import org.eclipse.jdt.ui.text.java.IQuickFixProcessor;

// NOT_PUBLISHED
public class RegExpQuickFixProcessor implements IQuickFixProcessor {

  public boolean hasCorrections(final ICompilationUnit unit, final int problemId) {
    for (final RegExpQuickFix quickFix : getQuickFixes()) {
      if (problemId == quickFix.getProblemId()) {
        return true;
      }
    }
    return false;
  }

  private List<RegExpQuickFix> getQuickFixes() {
    return PoorMansQuickFixesPlugin.getDefault().getRegExpQuickFixes();
  }

  public IJavaCompletionProposal[] getCorrections(
      final IInvocationContext context,
      final IProblemLocation[] locations) throws CoreException {
    if (locations == null || locations.length == 0) {
      return null;
    }
    final ArrayList<IJavaCompletionProposal> resultingCollections = new ArrayList<IJavaCompletionProposal>();
    for (final IProblemLocation problemLocation : locations) {
      process(context, problemLocation, resultingCollections);
    }
    return resultingCollections.toArray(new IJavaCompletionProposal[resultingCollections.size()]);
  }

  private void process(
      final IInvocationContext context,
      final IProblemLocation problem,
      final ArrayList<IJavaCompletionProposal> proposals) {
    if (problem.getProblemId() == 0) { // no proposals for none-problem locations
      return;
    }
    final String source;
    try {
      source = context.getCompilationUnit().getSource();
    }
    catch (final JavaModelException e) {
      //ignore
      return;
    }
    final int offset = problem.getOffset();
    final int length = problem.getLength();

    final String substring = source.substring(offset, offset + length);
    Ambience ambience = null;
    for (final RegExpQuickFix quickFix : getQuickFixes()) {
      if (problem.getProblemId() == quickFix.getProblemId()) {
        final Pattern pattern = quickFix.getPattern();
        final Matcher matcher = pattern.matcher(substring);
        if (matcher.matches()) {
          final Object[] groups = getAllGroups(matcher);
          final String replacement = MessageFormat.format(quickFix.getReplacementFormat(), groups);
          final String name = MessageFormat.format(quickFix.getNameFormat(), groups);
          if (ambience == null) {
            //lazy for performance reasons
            ambience = Ambience.createAmbience(source, offset, offset + length);
          }
          final int relevance = quickFix.getRelevance();
          proposals
              .add(new TextReplaceCompletionProposal(name, ambience, offset, length, replacement, relevance));
        }
      }
    }
  }

  private static String[] getAllGroups(final Matcher matcher) {
    final String[] groups = new String[matcher.groupCount() + 1];
    for (int i = 0; i < groups.length; ++i) {
      groups[i] = matcher.group(i);
    }
    return groups;
  }
}