package de.jave.eclipse.poormans.quickfixes.util;

import java.util.ArrayList;
import java.util.List;

public class ObjectModel<T> {

  private final List<IChangeListener> listeners = new ArrayList<IChangeListener>();

  private T value;

  public synchronized void addChangeListener(IChangeListener changeListener) {
    listeners.add(changeListener);
  }

  public synchronized void removeChangeListener(IChangeListener changeListener) {
    listeners.remove(changeListener);
  }

  private void fireChangeEvent() {
    ArrayList<IChangeListener> clonedListeners;
    synchronized (this) {
      clonedListeners = new ArrayList<IChangeListener>(listeners);
    }
    ChangeEvent event = new ChangeEvent(this);
    for (IChangeListener listener : clonedListeners) {
      listener.stateChanged(event);
    }
  }

  public ObjectModel() {
    this(null);
  }

  public ObjectModel(T value) {
    this.value = value;
  }

  public T getValue() {
    synchronized (this) {
      return value;
    }
  }

  public void setValue(T value) {
    synchronized (this) {
      if (equals(this.value, value)) {
        return;
      }
      this.value = value;
      fireChangeEvent();
    }
  }

  private final static boolean equals(Object o1, Object o2) {
    return (o1 == null && o2 == null) || (o1 != null && o1.equals(o2));
  }

  @Override
  public String toString() {
    return "ObjectModel{" + value + "}"; //$NON-NLS-1$ //$NON-NLS-2$
  }
}