package de.jave.eclipse.poormans.quickfixes.util;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jdt.core.compiler.IProblem;

public class ProblemIdFactory {

  public final static String getConstantName(int value) {
    List<ProblemIdConstant> constants = getProblemIdConstants();
    for (ProblemIdConstant constant : constants) {
      if (value == constant.getValue()) {
        return IProblem.class.getSimpleName() + "." + constant.getName(); //$NON-NLS-1$
      }
    }
    return null;
  }

  private final static boolean isStaticIntField(final Field field) {
    if (!Modifier.isStatic(field.getModifiers())) {
      return false;
    }
    return (int.class.isAssignableFrom(field.getType()));
  }

  public static List<ProblemIdConstant> getProblemIdConstants() {
    final List<ProblemIdConstant> constants = new ArrayList<ProblemIdConstant>();
    Field[] fields = IProblem.class.getFields();
    for (int i = 0; i < fields.length; ++i) {
      final Field field = fields[i];
      if (isStaticIntField(field)) {
        Integer intValue;
        try {
          intValue = (Integer) field.get(null);
        }
        catch (IllegalAccessException e) {
          throw new RuntimeException(e);
        }
        constants.add(new ProblemIdConstant(field.getName(), intValue.intValue()));
      }
    }
    Collections.sort(constants, new Comparator<ProblemIdConstant>() {
      public int compare(ProblemIdConstant o1, ProblemIdConstant o2) {
        return o1.getName().compareTo(o2.getName());
      }
    });
    return constants;
  }
}